/**
 * Vash - JavaScript Template Parser
 *
 * https://github.com/kirbysayshi/vash
 *
 * Copyright (c) 2011 Andrew Petersen
 * MIT License (LICENSE)
 */
(function(exports){

	exports["version"] = "0.2.2-256";

	exports["config"] = {
		 "useWith": false
		,"modelName": "model"
	};

	/************** Begin injected code from build script */
	
// This pattern and basic lexer code are taken from the Jade lexer:
// https://github.com/visionmedia/jade/blob/master/lib/lexer.js

function VLexer(str){
	this.tokens = [];
	this.input = str.replace(/\r\n|\r/g, '\n');
	this.deferredTokens = [];
	this.stash = [];
	this.lineno = 1;
	this.charno = 0;
}

VLexer.prototype = {
	
	tok: function(type, val){
		return {
			 type: type
			,line: this.lineno
			,chr: this.charno
			,val: val
		}
	}
	
	,scan: function(regexp, type){
		var captures, token;
	    if (captures = regexp.exec(this.input)) {
			this.consume(captures[0].length);
			
			token = this.tok(type, captures[1]);
			this.charno += captures[0].length;
			return token;
	    }
	}
	
	,spew: function(str){
		this.input = str + this.input;
		this.charno -= str.length;
	}

	,consume: function(len){
		this.input = this.input.substr(len);
	}

	,spewIf: function(tok, ifStr){
		var parts;

		if(tok){
			parts = tok.val.split(ifStr);

			if(parts.length > 1){
				tok.val = parts.shift();
				this.spew(ifStr + parts.join(ifStr));
			}
		}
		
		return tok;
	}

	,advance: function(){
		return this.deferred()
			|| this.stashed()
			|| this.next();
	}

	,defer: function(tok){
		this.deferredTokens.push(tok);
	}

	,lookahead: function(n){
		var fetch = n - this.stash.length;
		while (fetch-- > 0) this.stash.push(this.next());
		return this.stash[--n];
	}

	,next: function() {
		return this.EMAIL()
			|| this.AT_STAR_OPEN()
			|| this.AT_STAR_CLOSE()

			|| this.AT_COLON()
			|| this.AT()
			
			|| this.PAREN_OPEN()
			|| this.PAREN_CLOSE()
			
			|| this.HARD_PAREN_OPEN()
			|| this.HARD_PAREN_CLOSE()
			
			|| this.BRACE_OPEN()
			|| this.BRACE_CLOSE()
			
			|| this.TEXT_TAG_OPEN()
			|| this.TEXT_TAG_CLOSE()
			
			|| this.HTML_TAG_SELFCLOSE()
			|| this.HTML_TAG_OPEN()
			|| this.HTML_TAG_CLOSE()
			
			|| this.PERIOD()
			|| this.NEWLINE()
			|| this.WHITESPACE()
			|| this.KEYWORD()
			|| this.IDENTIFIER()
			|| this.CONTENT()
	}

	,deferred: function() {
		return this.deferredTokens.length 
			&& this.deferredTokens.shift();
	}

	,stashed: function() {
		return this.stash.length
			&& this.stash.shift();
	}
	
	,AT: function(){
		return this.scan(/^(@)/, VLexer.tks.AT);
	}
	,AT_COLON: function(){
		return this.scan(/^@\:/, VLexer.tks.AT_COLON);
	}
	,AT_STAR_OPEN: function(){
		return this.scan(/^(@\*)/, VLexer.tks.AT_STAR_OPEN);
	}
	,AT_STAR_CLOSE: function(){
		return this.scan(/^(\*@)/, VLexer.tks.AT_STAR_CLOSE);
	}
	,PAREN_OPEN: function(){
		return this.scan(/^(\()/, VLexer.tks.PAREN_OPEN);
	}
	,PAREN_CLOSE: function(){
		return this.scan(/^(\))/, VLexer.tks.PAREN_CLOSE);
	}
	,BRACE_OPEN: function(){
		return this.scan(/^(\{)/, VLexer.tks.BRACE_OPEN);
	}
	,BRACE_CLOSE: function(){
		return this.scan(/^(\})/, VLexer.tks.BRACE_CLOSE);
	}
	,HARD_PAREN_OPEN: function(){
		return this.scan(/^(\[)/, VLexer.tks.HARD_PAREN_OPEN);
	}
	,HARD_PAREN_CLOSE: function(){
		return this.scan(/^(\])/, VLexer.tks.HARD_PAREN_CLOSE);
	}
	,TEXT_TAG_OPEN: function(){
		return this.scan(/^(<text>)/, VLexer.tks.TEXT_TAG_OPEN);
	}
	,TEXT_TAG_CLOSE: function(){
		return this.scan(/^(<\/text>)/, VLexer.tks.TEXT_TAG_CLOSE);
	}
	,HTML_TAG_SELFCLOSE: function(){
		return this.spewIf(this.scan(/^(<[^>]+?\/>)/, VLexer.tks.HTML_TAG_SELFCLOSE), '@');
	}
	,HTML_TAG_OPEN: function(){
		return this.spewIf(this.scan(/^(<[^\/ >]+?[^>]*?>)/, VLexer.tks.HTML_TAG_OPEN), '@');
	} 
	,HTML_TAG_CLOSE: function(){
		return this.spewIf(this.scan(/^(<\/[^>\b]+?>)/, VLexer.tks.HTML_TAG_CLOSE), '@');
	}
	,KEYWORD: function(){
		return this.scan(/^(case|catch|do|else|finally|for|function|goto|if|instanceof|return|switch|try|typeof|var|while|with)/, VLexer.tks.KEYWORD);
	}
	,IDENTIFIER: function(){
		return this.scan(/^([_$a-zA-Z\xA0-\uFFFF][_$a-zA-Z0-9\xA0-\uFFFF]*)/, VLexer.tks.IDENTIFIER);
	}
	,PERIOD: function(){
		return this.scan(/^(\.)/, VLexer.tks.PERIOD);
	}
	,EMAIL: function(){
		return this.scan(/^([a-zA-Z0-9._%-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,4})\b/, VLexer.tks.EMAIL);
	}
	,CONTENT: function(){
		return this.scan(/^([^\s})@.]+?)/, VLexer.tks.CONTENT);
	}
	,WHITESPACE: function(){
		return this.scan(/^(\s+)/, VLexer.tks.WHITESPACE);
	}
	,NEWLINE: function(){
		var token = this.scan(/^(\n)/, VLexer.tks.NEWLINE);
		if(token){
			this.lineno++;
			this.charno = 0;
		}
		return token;
	}
}

VLexer.tks = {
	 AT: 'AT'
	,AT_STAR_OPEN: 'AT_STAR_OPEN'
	,AT_STAR_CLOSE: 'AT_STAR_CLOSE'
	,AT_COLON: 'AT_COLON'
	,EMAIL: 'EMAIL'
	,PAREN_OPEN: 'PAREN_OPEN'
	,PAREN_CLOSE: 'PAREN_CLOSE'
	,BRACE_OPEN: 'BRACE_OPEN'
	,BRACE_CLOSE: 'BRACE_CLOSE'
	,HARD_PAREN_OPEN: 'HARD_PAREN_OPEN'
	,HARD_PAREN_CLOSE: 'HARD_PAREN_CLOSE'
	,TEXT_TAG_OPEN: 'TEXT_TAG_OPEN'
	,TEXT_TAG_CLOSE: 'TEXT_TAG_CLOSE'
	,HTML_TAG_SELFCLOSE: 'HTML_TAG_SELFCLOSE'
	,HTML_TAG_OPEN: 'HTML_TAG_OPEN'
	,HTML_TAG_CLOSE: 'HTML_TAG_CLOSE'
	,KEYWORD: 'KEYWORD'
	,IDENTIFIER: 'IDENTIFIER'
	,PERIOD: 'PERIOD'
	,CONTENT: 'CONTENT'
	,WHITESPACE: 'WHITESPACE'
	,NEWLINE: 'NEWLINE'
};


function Stack(){
	this._stack = []
}

Stack.prototype = {
	push: function(obj){
		this._stack.push(obj);
		return this;
	}
	,pop: function(){
		if(this._stack.length > 0)
			return this._stack.pop();
		else 
			throw new Error('Stack Underflow');
	}
	,peek: function(){
		if(this._stack.length > 0){
			return this._stack[ this._stack.length - 1 ]
		} else {
			return null;
		}
	}
	,count: function(){
		return this._stack.length;
	}
	,raw: function(){
		return this._stack;
	}
};

function VParser(str){
	if(typeof str !== 'string')
		throw new VParser.exceptions.INVALIDINPUT(str);
	this.lex = new VLexer(str);
	this.tks = VLexer.tks;
	
	this.blockStack = new Stack();
	this.mode = VParser.modes.MKP;
	
	this.buffer = '';
	this.buffers = [];

	this.debug = false;
}

VParser.modes = { MKP: "MARKUP", BLK: "BLOCK", EXP: "EXPRESSION" };

VParser.exceptions = (function(){

	var err = {
		UNMATCHED: function UNMATCHED(tok){
			this.name = "UnmatchedCharacterError";
			this.message = 'Unmatched ' + tok.type
				+ ' at line ' + tok.line
				+ ', character ' + tok.chr
				+ '. Value: ' + tok.val
			this.lineNumber = tok.line;
			this.stack = '';
		}
		,INVALIDINPUT: function INVALIDINPUT(input){
			this.name = "InvalidParserInputError";
			this.message = 'Asked to parse invalid or non-string input: '
				+ input;
			this.lineNumber = 0;
			this.stack = '';
		}
	};
	
	for(var e in err){
		if(err.hasOwnProperty(e)){
			err[e].prototype = new Error();
			err[e].constructor = err[e];
		}
	}
	
	return err;
})();

VParser.prototype = {

	parse: function(){
		var curr, i, len, block;
		
		while( (curr = this.lex.advance()) ){
			this.debug && console.debug(curr.val, curr.type, curr);
			
			if(this.mode === VParser.modes.MKP){
				this._handleMKP(curr);
				continue;
			}
			
			if(this.mode === VParser.modes.BLK){
				this._handleBLK(curr);
				continue;
			}
			
			if(this.mode === VParser.modes.EXP){
				this._handleEXP(curr);	
				continue;
			}
		}

		this._endMode(VParser.modes.MKP);

		for(i = 0, len = this.blockStack.count(); i < len; i++){
			block = this.blockStack.pop();
			
			// only throw errors if there is an unclosed block
			if(block.type === VParser.modes.BLK)
				throw new VParser.exceptions.UNMATCHED(block.tok);
		}
			
		
		return this.buffers;
	}
	
	,compile: function(options){
		options = options || {};
		options.useWith = options.useWith === true ? true : false;
		options.modelName = options.modelName || 'model';
	
		var	 i
			,len
			,previous = null
			,current = null
			,generated = 'var out = "";\n'
			,modes = VParser.modes
			,reQuote = /[\"']/gi
			,reLineBreak = /[\n\r]/gi;

		for(i = 0, len = this.buffers.length; i < len; i++){
			previous = current;
			current = this.buffers[i];

			if(current.type === modes.MKP){
				generated += 
					(previous !== null && (previous.type === modes.MKP || previous.type === modes.EXP) 
						? '+' 
						: 'out += ') 
					+ '\'' 
					+ current.value
						.replace(reQuote, '\"')
						.replace(reLineBreak, '\\n') 
					+ '\'\n';
			}

			if(current.type === modes.BLK){
				// Nuke new lines, otherwise causes parse error
				generated += current.value
					.replace(reQuote, '\"')
					.replace(reLineBreak, '') + '\n';
			}

			if(current.type === modes.EXP){
				generated += 
					(previous !== null && (previous.type === modes.MKP || previous.type === modes.EXP) 
						? '+' 
						: 'out +=') 
					+ ' (' 
					+ current.value
						.replace(reQuote, '\"')
						.replace(reLineBreak, '\\n') 
					+ ')\n';
			}
		}

		return new Function(options.modelName, 
			(options.useWith === true 
				? "with(" + options.modelName + " || {}){" + generated + "}" 
				: generated ) + "\nreturn out;");
	
	}
	
	,_useToken: function(tok){
		this.buffer += tok.val;
	}
	
	,_useTokens: function(toks){
		for(var i = 0, len = toks.length; i < len; i++){
			this.buffer += toks[i].val;
		}
	}
	
	,_advanceUntilMatched: function(curr, start, end){
		var next = curr
			,nstart = 0
			,nend = 0
			,tks = [];
		
		while(next){
			if(next.type === start) nstart++;
			if(next.type === end) nend++;
			
			tks.push(next);
			
			if(nstart === nend) break;
			next = this.lex.advance();
			if(!next) throw new VParser.exceptions.UNMATCHED(curr);
		}
		
		return tks;
	}
	
	,_endMode: function(nextMode){
		if(this.buffer !== ''){
			this.buffers.push( { type: this.mode, value: this.buffer } );
			this.buffer = '';
		}
		this.mode = nextMode || VParser.modes.MKP;
	}
	
	,_handleMKP: function(curr){
		var  next = this.lex.lookahead(1)
			,ahead = this.lex.lookahead(2)
			,block = null
			,tagName = null
			,tempStack = [];
		
		switch(curr.type){
			
			case this.tks.AT_STAR_OPEN:
				this._advanceUntilMatched(curr, this.tks.AT_STAR_OPEN, this.tks.AT_STAR_CLOSE);
				break;
			
			case this.tks.AT:
				if(next) switch(next.type){
					
					case this.tks.PAREN_OPEN:
					case this.tks.IDENTIFIER:
						this._endMode(VParser.modes.EXP);
						break;
					
					case this.tks.KEYWORD:
					case this.tks.BRACE_OPEN:
						this._endMode(VParser.modes.BLK);
						break;
					
					default:
						this._useToken(this.lex.advance());

						break;
				}
				break;		
			
			case this.tks.BRACE_CLOSE:
				this._endMode(VParser.modes.BLK);
				this.lex.defer(curr);
				break;
			
			case this.tks.TEXT_TAG_OPEN:
			case this.tks.HTML_TAG_OPEN:
				tagName = curr.val.match(/^<([^\/ >]+)/i); 
				
				if(tagName === null && next && next.type === this.tks.AT && ahead)
					tagName = ahead.val.match(/(.*)/); // HACK for <@exp>

				this.blockStack.push({ type: VParser.modes.MKP, tag: tagName[1], tok: curr });
				if(this.tks.HTML_TAG_OPEN === curr.type) this._useToken(curr);
				break;
			
			case this.tks.TEXT_TAG_CLOSE:
			case this.tks.HTML_TAG_CLOSE:
				tagName = curr.val.match(/^<\/([^>]+)/i); 
				
				if(tagName === null && next && next.type === this.tks.AT && ahead)
					tagName = ahead.val.match(/(.*)/); // HACK for </@exp>
				
				block = this.blockStack.pop();
				
				while(block !== null){
					if(block.type === VParser.modes.MKP && tagName[1] === block.tag){
						break;
					}
					tempStack.push(block);
					block = this.blockStack.pop();
				}
				
				if(block === null){
					// couldn't find opening tag
					throw new VParser.exceptions.UNMATCHED(block.tok);
				}
				
				// put all blocks back except for found
				this.blockStack.raw().push.apply(this.blockStack.raw(), tempStack);
				
				if(this.tks.HTML_TAG_CLOSE === curr.type) this._useToken(curr);

				block = this.blockStack.peek();
				if(block !== null && block.type === VParser.modes.BLK){
					this._endMode(VParser.modes.BLK);
				}
				break;
			
			default:
				this._useToken(curr);
				break;
		}
		
	}
	
	,_handleBLK: function(curr){
		
		var next = this.lex.lookahead(1)
			,block = null
			,tempStack = [];
		
		switch(curr.type){
			
			case this.tks.AT:
				switch(next.type){
					
					case this.tks.AT:
						break;
					
					default:
						this.lex.defer(curr);
						this._endMode(VParser.modes.MKP);
						break;
				}
				break;
			
			case this.tks.AT_COLON:
				this._endMode(VParser.modes.MKP);
				break;
			
			case this.tks.TEXT_TAG_OPEN:
			case this.tks.TEXT_TAG_CLOSE:
			case this.tks.HTML_TAG_SELFCLOSE:
			case this.tks.HTML_TAG_OPEN:
			case this.tks.HTML_TAG_CLOSE:
				this._endMode(VParser.modes.MKP);
				this.lex.defer(curr);
				break;
			
			case this.tks.BRACE_OPEN:
				this.blockStack.push({ type: VParser.modes.BLK, tok: curr });
				this._useToken(curr);
				break;
			
			case this.tks.BRACE_CLOSE:
				block = this.blockStack.pop();
				
				// try to find a block of type BLK. save non-BLKs for later...
				while(block !== null && block.type !== VParser.modes.BLK ){
					tempStack.push(block);
					block = this.blockStack.pop();
				}
				
				// put non-BLKs back in
				this.blockStack.raw().push.apply(this.blockStack.raw(), tempStack);
				
				if(block === null || (block !== null && block.type !== VParser.modes.BLK))
					throw new VParser.exceptions.UNMATCHED(curr);
				
				this._useToken(curr);
				
				block = this.blockStack.peek();
				if(block !== null && block.type === VParser.modes.MKP) 
					this._endMode(VParser.modes.MKP);
				break;
			
			default:
				this._useToken(curr);
				break;
		}
		
	}
	
	,_handleEXP: function(curr){
		
		var ahead = null;
		
		switch(curr.type){
			
			case this.tks.KEYWORD:		
				this._endMode(VParser.modes.BLK);
				this.lex.defer(curr);
				break
			
			case this.tks.IDENTIFIER:
				this._useToken(curr);		
				break;
			
			case this.tks.HARD_PAREN_OPEN:
				this._useTokens(this._advanceUntilMatched(curr, this.tks.HARD_PAREN_OPEN, this.tks.HARD_PAREN_CLOSE));
				break;
			
			case this.tks.PAREN_OPEN:
				this._useTokens(this._advanceUntilMatched(curr, this.tks.PAREN_OPEN, this.tks.PAREN_CLOSE));
				break;
			
			case this.tks.PERIOD:
				ahead = this.lex.lookahead(1);
				if(ahead && ahead.type === this.tks.IDENTIFIER)
					this._useToken(curr);
				else {
					this._endMode(VParser.modes.MKP);
					this.lex.defer(curr);
				}
				break;
			
			default:
				// assume end of expression
				this._endMode(VParser.modes.MKP);
				this.lex.defer(curr);
				break;
				
		}
		
	}
	
}

	/************** End injected code from build script */

	exports["VLexer"] = VLexer;
	exports["VParser"] = VParser;
	exports["compile"] = function tpl(markup, options){

		var  p = new VParser(markup)
			,cmp;

		options = options || {};
		options.useWith = typeof options.useWith === 'undefined' 
			? exports.config.useWith 
			: options.useWith;
		options.modelName = options.modelName || exports.config.modelName;

		p.parse();
		cmp = p.compile(options)

		// Express support
		return function render(locals){
			return cmp(locals);
		}
	};

})(typeof exports === 'undefined' ? this['vash'] = {} : exports);